# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import test_base

import a10_neutron_lbaas.a10_exceptions as a10_ex


class TestPools(test_base.UnitTestBase):

    def test_sanity(self):
        pass

    def test_create(self):
        methods = {
            'ROUND_ROBIN':
                self.a.last_client.slb.service_group.ROUND_ROBIN,
            'LEAST_CONNECTIONS':
                self.a.last_client.slb.service_group.LEAST_CONNECTION,
            'SOURCE_IP':
                self.a.last_client.slb.service_group.WEIGHTED_LEAST_CONNECTION,
        }
        protocols = {
            'TCP': self.a.last_client.slb.service_group.TCP,
            'UDP': self.a.last_client.slb.service_group.UDP,
        }
        persistences = [None, 'SOURCE_IP', 'HTTP_COOKIE', 'APP_COOKIE']
        listeners = [False, True]

        for p in protocols.keys():
            for m in methods.keys():
                for pers in persistences:
                    for listener in listeners:
                        self.a.reset_mocks()
                        saw_exception = False

                        pool = test_base.FakePool(p, m, pers, listener)
                        try:
                            self.a.pool.create(None, pool)
                        except a10_ex.UnsupportedFeature as e:
                            if pers == 'APP_COOKIE':
                                saw_exception = True
                            else:
                                raise e

                        self.print_mocks()

                        # (self.a.last_client.slb.service_group.create.
                        #     assert_called_with(
                        #     pool.id,
                        #     lb_method=methods[m],
                        #     protocol=protocols[p]))

                        if not saw_exception:
                            n = str(self.a.last_client.mock_calls).index(
                                'slb.service_group.create')
                            self.assertTrue(n >= 0)

                        if pers == 'SOURCE_IP':
                            (self.a.last_client.slb.template.
                                src_ip_persistence.create.
                                assert_called_with(pool.id))
                        elif pers == 'HTTP_COOKIE':
                            (self.a.last_client.slb.template.
                                cookie_persistence.create.
                                assert_called_with(pool.id))
                        elif pers == 'APP_COOKIE':
                            self.assertTrue(saw_exception)

    def test_update(self):
        old_pool = test_base.FakePool('TCP', 'LEAST_CONNECTIONS', None, True)
        pool = test_base.FakePool('TCP', 'ROUND_ROBIN', None, True)
        self.a.pool.update(None, old_pool, pool)
        self.print_mocks()
        self.a.last_client.slb.service_group.create(
            pool.id,
            lb_method=self.a.last_client.slb.service_group.ROUND_ROBIN,
            protocol=self.a.last_client.slb.service_group.TCP)

    def test_delete(self):
        members = [[], [test_base.FakeMember()]]
        hms = [None, test_base.FakeHM('PING')]
        persistences = [None, 'SOURCE_IP', 'HTTP_COOKIE']
        listeners = [False, True]

        for m in members:
            for hm in hms:
                for pers in persistences:
                    for lst in listeners:
                        self.a.reset_mocks()

                        pool = test_base.FakePool('TCP', 'ROUND_ROBIN',
                                                  pers, lst,
                                                  members=m,
                                                  hm=hm)
                        self.a.pool.delete(None, pool)

                        self.print_mocks()

                        (self.a.last_client.slb.service_group.delete.
                            assert_called_with(pool.id))

                        if pers == 'SOURCE_IP':
                            (self.a.last_client.slb.template.
                                src_ip_persistence.delete.
                                assert_called_with(pool.id))
                        elif pers == 'HTTP_COOKIE':
                            (self.a.last_client.slb.template.
                                cookie_persistence.delete.
                                assert_called_with(pool.id))



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import test_base

import a10_neutron_lbaas.a10_exceptions as a10_ex


class TestLB(test_base.UnitTestBase):

    def test_create(self):
        m = test_base.FakeLoadBalancer()
        self.a.lb.create(None, m)
        s = str(self.a.last_client.mock_calls)
        self.assertTrue('call.slb.virtual_server.create' in s)
        self.assertTrue('fake-lb-id-001' in s)
        self.assertTrue('5.5.5.5' in s)
        self.assertTrue('UP' in s)

    def test_create_with_listeners(self):
        pool = test_base.FakePool('HTTP', 'ROUND_ROBIN', None)
        m = test_base.FakeLoadBalancer()
        for x in [1, 2, 3]:
            z = test_base.FakeListener('TCP', 2222+x, pool=pool,
                                       loadbalancer=m)
            m.listeners.append(z)
        self.a.lb.create(None, m)
        s = str(self.a.last_client.mock_calls)
        self.assertTrue('call.slb.virtual_server.create' in s)
        self.assertTrue('fake-lb-id-001' in s)
        self.assertTrue('5.5.5.5' in s)
        self.assertTrue('UP' in s)
        self.assertTrue('vport.create' in s)
        for x in [1, 2, 3]:
            self.assertTrue(str(2222+x) in s)

    def test_update_down(self):
        m = test_base.FakeLoadBalancer()
        m.admin_state_up = False
        self.a.lb.update(None, m, m)
        s = str(self.a.last_client.mock_calls)
        self.assertTrue('call.slb.virtual_server.update' in s)
        self.assertTrue('fake-lb-id-001' in s)
        self.assertTrue('5.5.5.5' in s)
        self.assertTrue('DOWN' in s)

    def test_delete(self):
        m = test_base.FakeLoadBalancer()
        self.a.lb.delete(None, m)
        s = str(self.a.last_client.mock_calls)
        self.assertTrue('call.slb.virtual_server.delete' in s)
        self.assertTrue('fake-lb-id-001' in s)

    def test_refresh(self):
        try:
            self.a.lb.refresh(None, test_base.FakeModel())
        except a10_ex.UnsupportedFeature:
            pass

    def test_stats(self):
        self.a.lb.stats(None, test_base.FakeModel())
        self.print_mocks()
        self.a.last_client.slb.virtual_server.stats.assert_called_with(
            'fake-id-001')



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import test_base


def return_one(*args):
    return 1


def return_two(*args):
    return 2


class TestMembers(test_base.UnitTestBase):

    def set_count_1(self):
        self.a.member.openstack_manager._count = return_one

    def set_count_2(self):
        self.a.member.openstack_manager._count = return_two

    def test_get_ip(self):
        m = test_base.FakeMember(pool=mock.MagicMock())
        self.a.member._get_ip(None, m, False)
        self.a.openstack_driver.member._get_ip.assert_called_with(
            None, m, False)

    def test_get_name(self):
        m = test_base.FakeMember(pool=mock.MagicMock())
        z = self.a.member._get_name(m, '1.1.1.1')
        self.assertEqual(z, '_get-o_1_1_1_1_neutron')

    def test_count(self):
        self.a.member._count(None, test_base.FakeMember(pool=mock.MagicMock()))

    def test_create(self, admin_state_up=True):
        m = test_base.FakeMember(admin_state_up=admin_state_up,
                                 pool=mock.MagicMock())
        ip = self.a.member._get_ip(None, m, True)
        name = self.a.member._get_name(m, ip)
        self.a.member.create(None, m)

        self.a.last_client.slb.server.create.assert_called_with(name, ip)
        if admin_state_up:
            status = self.a.last_client.slb.UP
        else:
            status = self.a.last_client.slb.DOWN
        self.a.last_client.slb.service_group.member.create.assert_called_with(
            m.pool.id, name, m.protocol_port, status=status)

    def test_create_down(self):
        self.test_create(False)

    def test_update_down(self):
        m = test_base.FakeMember(False, pool=mock.MagicMock())
        ip = self.a.member._get_ip(None, m, True)
        name = self.a.member._get_name(m, ip)
        self.a.member.update(None, m, m)

        self.a.last_client.slb.service_group.member.update.assert_called_with(
            m.pool.id, name, m.protocol_port, self.a.last_client.slb.DOWN)

    def test_delete(self):
        m = test_base.FakeMember(False, pool=mock.MagicMock())
        ip = self.a.member._get_ip(None, m, True)

        self.set_count_1()
        self.a.member.delete(None, m)

        self.a.last_client.slb.server.delete(ip)

    def test_delete_count_gt_one(self):
        m = test_base.FakeMember(False, pool=mock.MagicMock())
        ip = self.a.member._get_ip(None, m, True)
        name = self.a.member._get_name(m, ip)

        self.set_count_2()
        self.a.member.delete(None, m)

        self.a.last_client.slb.service_group.member.delete.assert_called_with(
            m.pool.id, name, m.protocol_port)



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

import acos_client.errors as acos_errors

LOG = logging.getLogger(__name__)


class A10Context(object):

    def __init__(self, handler, openstack_context, openstack_lbaas_obj,
                 **kwargs):
        self.handler = handler
        self.openstack_driver = handler.openstack_driver
        self.a10_driver = handler.a10_driver
        self.openstack_context = openstack_context
        self.openstack_lbaas_obj = openstack_lbaas_obj
        self.device_name = kwargs.get('device_name', None)
        LOG.debug("A10Context obj=%s", openstack_lbaas_obj)

    def __enter__(self):
        self.get_tenant_id()
        if self.device_name:
            d = self.a10_driver.config.devices[self.device_name]
        else:
            d = self.a10_driver._select_a10_device(self.tenant_id)
        self.device_cfg = d
        self.client = self.a10_driver._get_a10_client(self.device_cfg)
        self.select_appliance_partition()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        try:
            self.client.session.close()
        except acos_errors.InvalidSessionID:
            pass

        if exc_type is not None:
            return False

    def get_tenant_id(self):
        if hasattr(self.openstack_lbaas_obj, 'tenant_id'):
            self.tenant_id = self.openstack_lbaas_obj.tenant_id
        else:
            self.tenant_id = self.openstack_lbaas_obj['tenant_id']

    def select_appliance_partition(self):
        # If we are not using appliance partitions, we are done.
        if self.device_cfg['v_method'].lower() != 'adp':
            return

        # Try to make the requested partition active
        name = self.tenant_id[0:13]
        try:
            self.client.system.partition.active(name)
            return
        except acos_errors.NotFound:
            pass

        # Create it if not found
        self.client.system.partition.create(name)
        self.client.system.partition.active(name)


class A10WriteContext(A10Context):

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is None:
            try:
                self.client.system.action.write_memory()
            except acos_errors.InvalidSessionID:
                pass

            for v in self.device_cfg.get('ha_sync_list', []):
                self.client.ha.sync(v['ip'], v['username'], v['password'])

        super(A10WriteContext, self).__exit__(exc_type, exc_value, traceback)


# class A10WriteStatusContext(A10WriteContext):

#     def __exit__(self, exc_type, exc_value, traceback):
#         if exc_type is None:
#             self.openstack_manager.active(self.openstack_context,
#                                           self.openstack_lbaas_obj.id)
#         else:
#             self.openstack_manager.failed(self.openstack_context,
#                                           self.openstack_lbaas_obj.id)

#         super(A10WriteStatusContext, self).__exit__(exc_type, exc_value,
#                                                     traceback)


class A10DeleteContextBase(A10WriteContext):

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is None:
            # self.openstack_manager.db_delete(self.openstack_context,
            #                                  self.openstack_lbaas_obj.id)
            self.partition_cleanup_check()

        super(A10DeleteContextBase, self).__exit__(exc_type, exc_value,
                                                   traceback)

    def remaining_root_objects(self):
        return 1

    def partition_cleanup_check(self):
        # If we are not using appliance partitions, we are done.
        if self.device_cfg['v_method'].lower() != 'adp':
            return

        n = self.remaining_root_objects()
        if n == 0:
            try:
                name = self.tenant_id[0:13]
                self.client.system.partition.delete(name)
            except Exception:
                LOG.error("A10Driver: partition cleanup failed; ignoring")



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import a10_neutron_lbaas.a10_exceptions as a10_ex

import acos_client.errors as acos_errors
import handler_base
import v2_context as a10


class LoadBalancerHandler(handler_base.HandlerBase):

    def _set(self, c, set_method, context, load_balancer):
        status = c.client.slb.UP
        if not load_balancer.admin_state_up:
            status = c.client.slb.DOWN

        set_method(load_balancer.id, load_balancer.address, status)

    def create(self, context, load_balancer):
        with a10.A10WriteStatusContext(self, context, load_balancer) as c:
            self._set(c, c.client.slb.virtual_server.create, context,
                      load_balancer)

            for listener in load_balancer.listeners:
                try:
                    self.a10_driver.listener._create(c, context, listener)
                except acos_errors.Exists:
                    pass

    def update(self, context, old_load_balancer, load_balancer):
        with a10.A10WriteStatusContext(self, context, load_balancer) as c:
            self._set(c, c.client.slb.virtual_server.update, context,
                      load_balancer)

    def delete(self, context, load_balancer):
        with a10.A10DeleteContext(self, context, load_balancer) as c:
            c.client.slb.virtual_server.delete(load_balancer.id)

    def refresh(self, context, lb_obj, force=False):
        raise a10_ex.UnsupportedFeature()

    def stats(self, context, lb_obj):
        with a10.A10Context(self, context, lb_obj) as c:
            try:
                r = c.client.slb.virtual_server.stats(lb_obj.id)
                return {
                    "bytes_in": r["virtual_server_stat"]["req_bytes"],
                    "bytes_out": r["virtual_server_stat"]["resp_bytes"],
                    "active_connections":
                        r["virtual_server_stat"]["cur_conns"],
                    "total_connections": r["virtual_server_stat"]["tot_conns"]
                }
            except Exception:
                return {
                    "bytes_in": 0,
                    "bytes_out": 0,
                    "active_connections": 0,
                    "total_connections": 0
                }



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import test_base


class TestA10Openstack(test_base.UnitTestBase):

    def test_sanity(self):
        pass

    def test_select(self):
        a = self.a._select_a10_device("first-token")
        self.a._select_a10_device("second-token")
        self.assertEqual(a, self.a._select_a10_device("first-token"))

    def test_verify(self):
        self.a._verify_appliances()



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

import acos_client.errors as acos_errors
import handler_base
import v1_context as a10

LOG = logging.getLogger(__name__)


class PoolHandler(handler_base.HandlerBase):

    def _model_type(self):
        return 'pool'

    def _get_hm(self, context, hm_id):
        return self.openstack_driver._pool_get_hm(context, hm_id)

    def _get_member(self, context, member_id):
        return self.openstack_driver._member_get(context, member_id)

    def _get_tenant_id(self, context, pool_id):
        return self.openstack_driver._pool_get_tenant_id(context, pool_id)

    def _get_vip_id(self, context, pool_id):
        return self.openstack_driver._pool_get_vip_id(context, pool_id)

    def _get_vip(self, context, vip_id):
        return self.openstack_driver.plugin.get_vip(context, vip_id)

    def _meta_name(self, pool):
        return self.meta(pool, 'name', pool['id'])

    def _set(self, set_method, c, context, pool):
        z = c.client.slb.service_group
        lb_methods = {
            'ROUND_ROBIN': z.ROUND_ROBIN,
            'LEAST_CONNECTIONS': z.LEAST_CONNECTION,
            'SOURCE_IP': z.WEIGHTED_LEAST_CONNECTION,
            'WEIGHTED_ROUND_ROBIN': z.WEIGHTED_ROUND_ROBIN,
            'WEIGHTED_LEAST_CONNECTION': z.WEIGHTED_LEAST_CONNECTION,
            'LEAST_CONNECTION_ON_SERVICE_PORT':
                z.LEAST_CONNECTION_ON_SERVICE_PORT,
            'WEIGHTED_LEAST_CONNECTION_ON_SERVICE_PORT':
                z.WEIGHTED_LEAST_CONNECTION_ON_SERVICE_PORT,
            'FAST_RESPONSE_TIME': z.FAST_RESPONSE_TIME,
            'LEAST_REQUEST': z.LEAST_REQUEST,
            'STRICT_ROUND_ROBIN': z.STRICT_ROUND_ROBIN,
            'STATELESS_SOURCE_IP_HASH': z.STATELESS_SOURCE_IP_HASH,
            'STATELESS_DESTINATION_IP_HASH': z.STATELESS_DESTINATION_IP_HASH,
            'STATELESS_SOURCE_DESTINATION_IP_HASH':
                z.STATELESS_SOURCE_DESTINATION_IP_HASH,
            'STATELESS_PER_PACKET_ROUND_ROBIN':
                z.STATELESS_PER_PACKET_ROUND_ROBIN,
        }
        protocols = {
            'HTTP': z.TCP,
            'HTTPS': z.TCP,
            'TCP': z.TCP,
            'UDP': z.UDP
        }
        args = {'service_group': self.meta(pool, 'service_group', {})}

        set_method(self._meta_name(pool),
                   protocol=protocols[pool['protocol']],
                   lb_method=lb_methods[pool['lb_method']],
                   axapi_args=args)

    def create(self, context, pool):
        with a10.A10WriteStatusContext(self, context, pool) as c:
            try:
                self._set(c.client.slb.service_group.create,
                          c, context, pool)
            except acos_errors.Exists:
                pass

    def update(self, context, old_pool, pool):
        with a10.A10WriteStatusContext(self, context, pool) as c:
            self._set(c.client.slb.service_group.update,
                      c, context, pool)

    def delete(self, context, pool):
        with a10.A10DeleteContext(self, context, pool) as c:
            for member in pool['members']:
                m = self._get_member(context, member)
                self.a10_driver.member._delete(c, context, m)

            for hm in pool['health_monitors_status']:
                z = self._get_hm(context, hm['monitor_id'])
                self.a10_driver.hm._delete(c, context, z)

            if 'vip_id' in pool and pool['vip_id'] is not None:
                vip = self._get_vip(context, pool['vip_id'])
                self.a10_driver.vip._delete(c, context, vip)

            c.client.slb.service_group.delete(self._meta_name(pool))

    def stats(self, context, pool_id):
        tenant_id = self._get_tenant_id(context, pool_id)
        pool = {'id': pool_id, 'tenant_id': tenant_id}
        with a10.A10Context(self, context, pool) as c:
            try:
                vip_id = self._get_vip(context, pool['id'])
                vip = self._get_vip(context, vip_id)
                name = self.meta(vip, 'vip_name', vip['id'])
                r = c.client.slb.virtual_server.stats(name)
                return {
                    "bytes_in": r["virtual_server_stat"]["req_bytes"],
                    "bytes_out": r["virtual_server_stat"]["resp_bytes"],
                    "active_connections":
                        r["virtual_server_stat"]["cur_conns"],
                    "total_connections": r["virtual_server_stat"]["tot_conns"]
                }
            except Exception as e:
                return {
                    "bytes_in": 0,
                    "bytes_out": 0,
                    "active_connections": 0,
                    "total_connections": 0
                }



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
import sys

LOG = logging.getLogger(__name__)


class A10Config(object):

    DEVICE_DEFAULTS = {
        "status": True,
        "autosnat": True,
        "api_version": "2.1",
        "v_method": "LSI",
        "max_instance": 5000,
        "use_float": False,
        "method": "hash"
    }

    def __init__(self):
        self.config_dir = os.environ.get(
            'A10_CONFIG_DIR',
            '/etc/neutron/services/loadbalancer/a10networks')
        self.config_path = os.path.join(self.config_dir, "config.py")

        real_sys_path = sys.path
        sys.path = [self.config_dir]
        try:
            import config
            self.config = config
            self.devices = {}
            for k, v in self.config.devices.items():
                if 'status' in v and not v['status']:
                    LOG.debug("status is False, skipping dev: %s", v)
                else:
                    self.devices[k] = v

                    # Figure out port and protocol
                    protocol = self.devices[k].get('protocol', 'https')
                    port = self.devices[k].get(
                        'port', {'http': 80, 'https': 443}[protocol])
                    self.devices[k]['protocol'] = protocol
                    self.devices[k]['port'] = port

                    # Device defaults
                    for dk, dv in self.DEVICE_DEFAULTS.items():
                        if dk not in self.devices[k]:
                            self.devices[k][dk] = dv
        finally:
            sys.path = real_sys_path

        LOG.debug("A10Config, devices=%s", self.devices)

    @property
    def verify_appliances(self):
        if hasattr(self.config, 'verify_appliances'):
            return self.config.verify_appliances
        else:
            return True



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import handler_base
import v2_context as a10


class HealthMonitorHandler(handler_base.HandlerBase):

    def _hm_name(self, hm):
        return hm.id[0:28]

    def _set(self, c, set_method, context, hm):
        hm_map = {
            'PING': c.client.slb.hm.ICMP,
            'TCP': c.client.slb.hm.TCP,
            'HTTP': c.client.slb.hm.HTTP,
            'HTTPS': c.client.slb.hm.HTTPS
        }

        hm_name = self._hm_name(hm)
        method = None
        url = None
        expect_code = None
        if hm.type in ['HTTP', 'HTTPS']:
            method = hm.http_method
            url = hm.url_path
            expect_code = hm.expected_codes

        set_method(hm_name, hm_map[hm.type],
                   hm.delay, hm.timeout, hm.max_retries,
                   method=method, url=url, expect_code=expect_code)

    def _create(self, c, context, hm):
        self._set(c, c.client.slb.hm.create, context, hm)

        if hm.pool:
            c.client.slb.service_group.update(
                hm.pool.id,
                health_monitor=self._hm_name(hm))

    def create(self, context, hm):
        with a10.A10WriteStatusContext(self, context, hm) as c:
            self._create(c, context, hm)

    def update(self, context, old_hm, hm):
        with a10.A10WriteStatusContext(self, context, hm) as c:
            self._set(c, c.client.slb.hm.update, context, hm)

            if hm.pool and hm.pool != old_hm.pool:
                c.client.slb.service_group.update(
                    hm.pool.id,
                    health_monitor=self._hm_name(hm))
            elif not hm.pool and old_hm.pool:
                c.client.slb.service_group.update(
                    old_hm.pool.id,
                    health_monitor='')

    def _delete(self, c, context, hm):
        c.client.slb.hm.delete(hm.id[0:28])

    def delete(self, context, hm):
        with a10.A10DeleteContext(self, context, hm) as c:
            if hm.pool:
                c.client.slb.service_group.update(hm.pool.id,
                                                  health_monitor="")
            self._delete(c, context, hm)



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import test_base
import test_handler_member


class TestPools(test_base.UnitTestBase):

    def fake_pool(self, protocol, method):
        return {
            'tenant_id': 'ten1',
            'id': 'id1',
            'protocol': protocol,
            'lb_method': method,
        }.copy()

    def test_create(self):
        methods = {
            'ROUND_ROBIN':
                self.a.last_client.slb.service_group.ROUND_ROBIN,
            'LEAST_CONNECTIONS':
                self.a.last_client.slb.service_group.LEAST_CONNECTION,
            'SOURCE_IP':
                self.a.last_client.slb.service_group.WEIGHTED_LEAST_CONNECTION,
        }
        protocols = {
            'TCP': self.a.last_client.slb.service_group.TCP,
            'UDP': self.a.last_client.slb.service_group.UDP,
        }

        for p in protocols.keys():
            for m in methods.keys():
                self.a.reset_mocks()
                saw_exception = False

                pool = self.fake_pool(p, m)
                self.a.pool.create(None, pool)

                self.print_mocks()

                # (self.a.last_client.slb.service_group.create.
                #     assert_called_with(
                #     pool.id,
                #     lb_method=methods[m],
                #     protocol=protocols[p]))

                if not saw_exception:
                    n = str(self.a.last_client.mock_calls).index(
                        'slb.service_group.create')
                    self.assertTrue(n >= 0)

    def test_update(self):
        old_pool = self.fake_pool('TCP', 'LEAST_CONNECTIONS')
        pool = self.fake_pool('TCP', 'ROUND_ROBIN')
        self.a.pool.update(None, old_pool, pool)
        self.print_mocks()
        self.a.last_client.slb.service_group.create(
            pool['id'],
            lb_method=self.a.last_client.slb.service_group.ROUND_ROBIN,
            protocol=self.a.last_client.slb.service_group.TCP)

    def test_delete(self):
        pool = self.fake_pool('TCP', 'LEAST_CONNECTIONS')
        pool['members'] = [test_handler_member._fake_member()]
        pool['health_monitors_status'] = [{'monitor_id': 'hm1'}]
        self.a.pool.delete(None, pool)

        self.print_mocks()

        (self.a.last_client.slb.service_group.delete.
            assert_called_with(pool['id']))

    def test_stats(self):
        pool = self.fake_pool('TCP', 'LEAST_CONNECTIONS')
        z = self.a.pool
        z._get_tenant_id = lambda x, y: 'hello'
        z._get_vip_id = lambda x, y: '2.2.2.2'
        z.stats(None, pool['id'])
        self.print_mocks()
        s = str(self.a.last_client.mock_calls)
        self.assertTrue(s.index('slb.virtual_server.stats') >= 0)



# Copyright 2014, Doug Wiegley (dougwig), A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import test_base


class TestA10Config(test_base.UnitTestBase):

    def test_verify_appliances(self):
        self.assertTrue(self.a.config.verify_appliances)

    def test_num_appliances(self):
        self.assertEqual(3, len(self.a.config.devices))

    def test_expected_ports(self):
        self.assertEqual(8443, self.a.config.devices['ax1']['port'])
        self.assertEqual(80, self.a.config.devices['ax3']['port'])
        self.assertEqual(443, self.a.config.devices['ax4']['port'])

    def test_expected_protocols(self):
        self.assertEqual('https', self.a.config.devices['ax1']['protocol'])
        self.assertEqual('http', self.a.config.devices['ax3']['protocol'])
        self.assertEqual('https', self.a.config.devices['ax4']['protocol'])

    def test_v_method(self):
        for k, v in self.a.config.devices.items():
            self.assertEqual('LSI', v['v_method'].upper())
